/* re-size functions */

#include <qdos.h>
#include <qptr.h>
#include "spv.h"
#include "qspv-proto.h"

/* general re-size function - returns error from re-draw */

static void scale (struct WM_swdef *sw, char sc[], short dx, short dy)
{

    if (sc[0])
	sw->xsize += (dx * sc[0]) >> 3;
    if (sc[1])
	sw->ysize += (dy * sc[1]) >> 3;
    if (sc[2])
	sw->xorg += (dx * sc[2]) >> 3;
    if (sc[3])
	sw->yorg += (dy * sc[3]) >> 3;
}

struct WM_wsiz *GetLimits (struct WM_wwork *wwk)
{
    short nlx,nly;
    int sts;
    static struct WM_wsiz wl;

    sts = iop_flim (wwk->chid, -1, &wl);
    nlx = ((((wl.xsize - 14) / 8) - 1) * 8) + 14;
    nly = 40 + ((((wl.ysize - 40) / 12) - 1) * 12);
    
    wwk->wscale->xmax = nlx;
    wwk->wscale->ymax = nly;
    return &wl;
}

void re_draw(WM_wwork_t *wwk, short dx, short dy)
{
    wm_unset (wwk);  /* unset window because we are about to pr_pos */
    wwk->xorg = wwk->wstat->ptpsx - wwk->xorg;
    wwk->yorg = wwk->wstat->ptpsy - wwk->yorg;
    wm_prpos (wwk, (short) (wwk->wstat->ptpsx - dx),
	      (short) (wwk->wstat->ptpsy - dy));
    wm_wdraw (wwk);
}


long re_size (struct WM_wwork *wwk, short lit_size)
{
    static short flag = 0, ix, iy;
    short dx, dy, last = 0;
    int sts;
    short res = 0;

    wwk->wstat->litem[lit_size] = WSI_MKAV;	/* item available when we redraw */

    GetLimits (wwk);
    if (wwk->wstat->kprs == K_HIT)
    {
	sts = wm_chwin (wwk, &dx, &dy);		/* read pointer move for size event */
	dx = -dx;
	dy = -dy;		/* our true sense of change size */
	last = 1;
    }
    else
    {
	flag ^= 1;
	if (flag)
	{
	    dx = wwk->wscale->xmax - wwk->xsize;
	    dy = wwk->wscale->ymax - wwk->ysize;
	    ix = wwk->xsize;
	    iy = wwk->ysize;
	}
	else
	{
	    dx = ix - wwk->xsize;
	    dy = iy - wwk->ysize;
	}
    }

    res = re_scale (wwk, &dx, &dy);

    if (res)
    {
	SetWinFileParams(wwk, NULL, NULL, 0);
	re_draw(wwk, dx, dy);
	if (last)
	{
	    ix = wwk->xsize;
	    iy = wwk->ysize;
	}
    }
    else
    {
        wm_ldraw(wwk, -1);
    }
    return (long)res;
}

/* general re-scale function using the wscale structure: resets actual size
 * change in *pdx, *pdy. */

short re_scale (struct WM_wwork *wwk, short *pdx, short *pdy)
{
    short newsx;
    short newsy;
    int iscale;
    short res = 0;
    struct WM_wscale *sc;
    struct WM_infw *infw;
    struct WM_info *info;
    struct WM_litm *litm;
    struct WM_appl *appl;
    struct WM_appw *appw;

    sc = wwk->wscale;

    newsx = wwk->xsize + ((*pdx + sc->xinc / 2) / sc->xinc) * sc->xinc;
    /* adjust by discrete increments */

    if (newsx > sc->xmax)
	newsx = sc->xmax;	/* limit size */
    else if (newsx < sc->xmin)
	newsx = sc->xmin;

    *pdx = newsx - wwk->xsize;	/* the actual change */

    newsy = wwk->ysize + ((*pdy + sc->yinc / 2) / sc->yinc) * sc->yinc;
    /* adjust by discrete increments */

    if (newsy > sc->ymax)
	newsy = sc->ymax;	/* limit size */
    else if (newsy < sc->ymin)
	newsy = sc->ymin;

    *pdy = newsy - wwk->ysize;	/* the actual change */

    if (*pdx || *pdy)
    {
	res = 1;
        wwk->xsize = newsx;		/* set size */
        wwk->ysize = newsy;		/* set size */

	iscale = 0;		/* now follows a table of scale factors */

	if (infw = wwk->pinfo)
	    while (infw->xsize >= 0)
	    {
		scale ((struct WM_swdef *) infw, sc->s8[iscale++], *pdx, *pdy);	/* scale info window */

		if (info = (infw++)->pobl)
		    while (info->xsize >= 0)
		    {
			scale ((struct WM_swdef *) info, sc->s8[iscale++], *pdx, *pdy);	/* and objects */
                        info++;
		    }
	    }

	if (litm = wwk->plitm)
	    while (litm->xsize >= 0)
	    {
		scale ((struct WM_swdef *) litm, sc->s8[iscale++], *pdx, *pdy);	/* scale loose item */
                litm++;
	    }
	if (appl = wwk->pappl)
	    while (appw = (appl++)->pappw)
		scale ((struct WM_swdef *) appw, sc->s8[iscale++], *pdx, *pdy);	/* scale app sub-window */
    }
    return res;
}




